@@ -1675,6 +1675,166 @@ fn test_decnum_tryinto_primitive() {
     quantizable(9999, 99999999, false);
 }
 
+#[test]
+fn test_decnum_tryinto_primitive_f32() {
+    const WIDTH: usize = 14;
+    fn inner(s: &str, e: Result<f32, ()>) {
+        let mut cx = Context::<Decimal<WIDTH>>::default();
+        let d = cx.parse(s).unwrap();
+        let r = cx.try_into_f32(d);
+        if e.is_err() {
+            assert!(r.is_err(), "expected {:?}, got {:?}", e, r)
+        } else {
+            let e = e.unwrap();
+            let r = r.unwrap();
+            if e.is_nan() {
+                assert!(r.is_nan())
+            } else {
+                assert_eq!(e, r)
+            }
+        }
+    }
+    inner("1.234", Ok(1.234));
+    inner("1234.567891234567", Ok(1234.5679));
+    inner(
+        "0.0000000000000001234567891234567",
+        Ok(0.00000000000000012345679),
+    );
+    inner(
+        "-0.0000000000000001234567891234567",
+        Ok(-0.00000000000000012345679),
+    );
+    inner("0.000", Ok(0.0));
+    inner("NaN", Ok(f32::NAN));
+    inner("-NaN", Ok(f32::NAN));
+    inner("Infinity", Ok(f32::INFINITY));
+    inner("-Infinity", Ok(f32::NEG_INFINITY));
+    inner("1000", Ok(1000.0));
+    inner(&f32::MAX.to_string(), Ok(f32::MAX));
+    inner(&f32::MIN.to_string(), Ok(f32::MIN));
+    inner(&f64::MAX.to_string(), Err(()));
+    inner(&f64::MIN.to_string(), Err(()));
+    inner("1e39", Err(()));
+    inner("1e-50", Err(()));
+}
+
+#[test]
+fn test_decnum_tryinto_primitive_f64() {
+    const WIDTH: usize = 14;
+    fn inner(s: &str, f: Result<f64, ()>) {
+        let mut cx = Context::<Decimal<WIDTH>>::default();
+        let d = cx.parse(s).unwrap();
+        let r = cx.try_into_f64(d);
+
+        if f.is_err() {
+            assert!(r.is_err(), "expected {:?}, got {:?}", f, r)
+        } else {
+            let f = f.unwrap();
+            let r = r.unwrap();
+            if f.is_nan() {
+                assert!(f.is_nan())
+            } else {
+                assert_eq!(f, r)
+            }
+        }
+    }
+    inner("1.234", Ok(1.234));
+    inner(
+        "1234567890.12345678901234567891234567890",
+        Ok(1234567890.1234568),
+    );
+    inner(
+        "12345678912345678901.234567891234567890",
+        Ok(12345678912345680000.0),
+    );
+    inner(
+        ".0000000000000000000000012345678912345678901234567891234567890",
+        Ok(0.000000000000000000000001234567891234568),
+    );
+    inner(
+        "1.2345678912345678901234567891234567890E-24",
+        Ok(0.000000000000000000000001234567891234568),
+    );
+    inner(
+        "-1.2345678912345678901234567891234567890E-24",
+        Ok(-0.000000000000000000000001234567891234568),
+    );
+    inner("1e39", Ok(1000000000000000000000000000000000000000.0));
+    inner("0.000", Ok(0.0));
+    inner(&f64::MAX.to_string(), Ok(f64::MAX));
+    inner(&f64::MIN.to_string(), Ok(f64::MIN));
+    inner("NaN", Ok(f64::NAN));
+    inner("-NaN", Ok(f64::NAN));
+    inner("Infinity", Ok(f64::INFINITY));
+    inner("-Infinity", Ok(f64::NEG_INFINITY));
+    inner("1e500", Err(()));
+    inner("1e-500", Err(()));
+}
+
+#[test]
+fn test_decnum_try_from_f32() {
+    const WIDTH: usize = 12;
+    fn inner(f: f32, e: &str) {
+        let mut cx = Context::<Decimal<WIDTH>>::default();
+        let d = cx.from_f32(f);
+        let e = cx.parse(e).unwrap();
+        if e.is_nan() {
+            assert!(d.is_nan());
+        } else {
+            assert_eq!(d, e);
+        }
+    }
+    inner(0f32, "0");
+    inner(0.0f32, "0");
+    inner(1f32, "1");
+    inner(101.101, "101.101");
+    inner(-101.101, "-101.101");
+    inner(f32::MAX, &f32::MAX.to_string());
+    inner(f32::MIN, &f32::MIN.to_string());
+    inner(f32::INFINITY, "infinity");
+    inner(f32::NEG_INFINITY, "-infinity");
+    inner(f32::NAN, "NaN");
+    inner(1.2345e-45f32, "1E-45");
+    inner(
+        12345678901234567890123456789012345678f32,
+        "1.23456780000000000000000000000000000E+37",
+    );
+}
+
+#[test]
+fn test_decnum_try_from_f64() {
+    const WIDTH: usize = 12;
+    fn inner(f: f64, e: &str) {
+        println!("f {}", f);
+        let mut cx = Context::<Decimal<WIDTH>>::default();
+        let d = cx.from_f64(f);
+        let e = cx.parse(e).unwrap();
+        if e.is_nan() {
+            assert!(d.is_nan());
+        } else {
+            assert_eq!(d, e);
+        }
+    }
+    inner(0f64, "0");
+    inner(0.0f64, "0");
+    inner(1f64, "1");
+    inner(101.101, "101.101");
+    inner(-101.101, "-101.101");
+    inner(f64::MAX, &f64::MAX.to_string());
+    inner(f64::MIN, &f64::MIN.to_string());
+    inner(f64::INFINITY, "infinity");
+    inner(f64::NEG_INFINITY, "-infinity");
+    inner(f64::NAN, "NaN");
+    inner(1.2345e-45f64, "1.2345E-45");
+    inner(
+        12345678901234567890123456789012345678901234567890123456789012345678901234567890f64,
+        "1.23456789012345680000000000000000000E+79",
+    );
+    inner(9.8765431E-300, "9.8765431E-300");
+    inner(9.8765431E-400, "0");
+    inner(9.8765431E305, "9.87654310000000000000000000000000000E+305");
+}
+
 #[test]
 fn test_decnum_coefficient() {
     const WIDTH: usize = 14;
@@ -1757,3 +1917,35 @@ fn test_decnum_coefficient() {
     inner(&min_i128);
     inner(&max_i128);
 }
+
+#[test]
+fn decnum_raw_parts() {
+    fn inner(s: &str, o: Option<i128>) {
+        const N: usize = 13;
+        let mut cx = Context::<Decimal<N>>::default();
+        let d = cx.parse(s).unwrap();
+        let (digits, exponent, bits, lsu) = d.to_raw_parts();
+        let r = unsafe { Decimal::<N>::from_raw_parts(digits, exponent, bits, &lsu) };
+        if d.is_nan() {
+            assert!(r.is_nan())
+        } else {
+            assert_eq!(d, r);
+        }
+        if let Some(o) = o {
+            let o = cx.from_i128(o);
+            assert_eq!(o, d);
+        }
+    }
+    inner("1", Some(1));
+    inner("-1", Some(-1));
+    inner("0.00", Some(0));
+    inner("987654321", Some(987654321));
+    inner("-987654321", Some(-987654321));
+    inner(&i128::MAX.to_string(), Some(i128::MAX));
+    inner(&i128::MIN.to_string(), Some(i128::MIN));
+    inner("98765.4321", None);
+    inner("-98765.4321", None);
+    inner("Infinity", None);
+    inner("-Infinity", None);
+    inner("NaN", None);
+}
